#!/usr/bin/perl
use List::Util qw/sum/;

die "should specify log file name" if ($#ARGV != 0);
$input_file = $ARGV[0];
$fieldlen = 8;
$naverage = 5;

$nsamples = -1;
$state = 0;
@codes = ();
open IN, '<', $input_file || die "failed to open log.txt";
while (<IN>) {
	chomp();
	next if (/^$/);

	if ($state == 0) {
		if (/^LABEL:(.*)$/) {
			$test_label = $1;
			die "label parse failed: $test_label"
				if ($test_label !~ /^(.*)-([0-9])(_([0-9]))?$/);
			$code=$1;
			$test=$2;
			$test_sub = (defined $4 ? $4 : "0");
			@time = ();
			$state = 1;
		}
	} elsif ($state == 1) {
		if (/^real\s+0m([0-9.]+)s$/) {
			push(@time, $1);
		} elsif (!/^(user|sys)/) {
			# record sample number of the first test
			if ($nsamples < 0) {
				$nsamples = $#time + 1;
				die "no sample found" if ($nsamples < 1);
				if ($nsamples <= $naverage) {
					print "WARNING: insufficient samples (n = $nsamples)\n";
				}

			}
			elsif ($#time != $nsamples - 1) {
				die "$.: $test_label: ". ($#time + 1) .
					" samples found but $nsamples expected.\n";
			}
			@time = sort {$a <=> $b} @time;

			# take the average of the fastest but stable 5 numbers
			if ($nsamples > $naverage) {
				$t = sum(@time[0..$naverage - 1]) / $naverage;
			} else {
				# too small sample, just take average
				$t = sum(@time) / ($#time + 1);
			}
			$result{"${test}_${test_sub}"}{$code} = $t;

			if (scalar(grep { $_ eq $code } @codes) == 0) {
				push(@codes, $code);
			}
				
			@time = ();

			$state = 0;
			redo if (/^LABEL:/);
		}
	}
}
close IN;

#
# print result table
#

# column title
printf " " x 5 . "|";
for $code (@codes) {
	$splen = int((($fieldlen - length($code)) / 2));
	$c = " " x $splen . $code.  " " x ($splen + 1);
	$c = substr($c, 0, $fieldlen);
	printf $c. "|";
}
print "\n";

# separator
printf "-" x 5 . "|";
for $code (@codes) {
	printf "-" x $fieldlen . "+"
}
print "\n";

# table body
for $test (sort keys %result) {
	printf "%4s |", $test;
	for $code (@codes) {
		if ($code ne "master") {
			$number = $result{$test}{$code};
		} else {
			# master cannot perform pruning test. duplicate non-pruning number
			if ($result{$test}{$code} > 0) {
				$number = $lastmaster = $result{$test}{$code};
			} else {
				$number = $lastmaster;
			}
		}
		printf " %6.3f |", $number
	}
	print "\n";
}
